@@ -29,7 +29,7 @@ module Data.Vector.Fusion.Stream (
   empty, singleton, cons, snoc, replicate, generate, (++),
 
   -- * Accessing individual elements
-  head, last, (!!),
+  head, last, (!!), (!?),
 
   -- * Substreams
   slice, init, tail, take, drop,
@@ -55,7 +55,7 @@ module Data.Vector.Fusion.Stream (
   and, or,
 
   -- * Unfolding
-  unfoldr, unfoldrN,
+  unfoldr, unfoldrN, iterateN,
 
   -- * Scans
   prescanl, prescanl',
@@ -199,11 +199,18 @@ last :: Stream a -> a
 {-# INLINE last #-}
 last = unId . M.last
 
+infixl 9 !!
 -- | Element at the given position
 (!!) :: Stream a -> Int -> a
 {-# INLINE (!!) #-}
 s !! i = unId (s M.!! i)
 
+infixl 9 !?
+-- | Element at the given position or 'Nothing' if out of bounds
+(!?) :: Stream a -> Int -> Maybe a
+{-# INLINE (!?) #-}
+s !? i = unId (s M.!? i)
+
 -- Substreams
 -- ----------
 
@@ -418,6 +425,11 @@ unfoldrN :: Int -> (s -> Maybe (a, s)) -> s -> Stream a
 {-# INLINE unfoldrN #-}
 unfoldrN = M.unfoldrN
 
+-- | Apply function n-1 times to value. Zeroth element is original value.
+iterateN :: Int -> (a -> a) -> a -> Stream a
+{-# INLINE iterateN #-}
+iterateN = M.iterateN
+
 -- Scans
 -- -----
 
