@@ -1,3 +1,8 @@
+#if __GLASGOW_HASKELL__ >= 703
+{-# LANGUAGE Safe #-}
+#elif __GLASGOW_HASKELL__ >= 701
+{-# LANGUAGE Trustworthy #-}
+#endif
 -- |
 -- Module      : Data.Vector.Fusion.Util
 -- Copyright   : (c) Roman Leshchinskiy 2009
@@ -13,7 +18,7 @@
 module Data.Vector.Fusion.Util (
   Id(..), Box(..),
 
-  delay_inline
+  delay_inline, delayed_min
 ) where
 
 -- | Identity monad
@@ -41,4 +46,8 @@ delay_inline :: (a -> b) -> a -> b
 {-# INLINE [0] delay_inline #-}
 delay_inline f = f
 
+-- | `min` inlined in phase 0
+delayed_min :: Int -> Int -> Int
+{-# INLINE [0] delayed_min #-}
+delayed_min m n = min m n
 
